@@ -101,3 +101,71 @@ class StrideManager(object):
         else:
             self.nb_keepers=0
 
+
+
+def is_trainable(name=None,index=None,matches=[],indices=[],searches=[],excludes=[]):
+    if name:
+        if name in matches:
+            return True
+        else:
+            found=next((s for s in searches if s in name),False)
+            if found:
+                exclude=next((e for e in excludes if e in name),False)
+                if not exclude:
+                    return True
+    if index and indices:
+        return index in indices
+    
+
+def set_trainable(
+        model,
+        matches=[],
+        indices=[],
+        searches=[],
+        excludes=[],
+        return_trainables=False,
+        noisy=False):
+    if matches or indices or searches:
+        matches=_as_list(matches)
+        indices=_as_list(indices)
+        searches=_as_list(searches)
+        excludes=_as_list(excludes)
+        indices=[int(i) for i in indices]
+        if return_trainables:
+            trainables=[]
+        for i,l in enumerate(model.layers):
+            l.trainable=is_trainable(
+                name=l.name,
+                index=i,
+                matches=matches,
+                indices=indices,
+                searches=searches,
+                excludes=excludes)
+            if return_trainables:
+                trainables.append(l)
+            if noisy and l.trainable:
+                print(l.name)
+        if return_trainables:
+            return trainables
+
+
+
+#
+# INTERNAL
+#
+def _as_list(value):
+    if value in [None,False]:
+        value=[]
+    elif not isinstance(value,list):
+        value=[value]
+    return value
+
+
+
+
+
+
+
+
+
+
